import type { MapObject } from '@/application/types'
import { defineStore } from 'pinia'

export type TeleportSettings = {
  toMapId: string
  toPositionX: number
  toPositionY: number
  toRotation: number
}

export const useMapEditorStore = defineStore('mapEditor', {
  state: () => {
    return {
      active: false,
      tool: 'move',
      drawMode: 'tile',
      eraserMode: 'tile',
      selectedTile: '',
      selectedMapObject: null as MapObject | null,
      isTileListModalShown: false,
      isMapObjectListModalShown: false,
      isMapListModalShown: false,
      isCreateMapModalShown: false,
      isSettingsModalShown: false,
      shouldClearTiles: false,
      teleportSettings: {
        toMapId: '',
        toPositionX: 0,
        toPositionY: 0,
        toRotation: 0
      } as TeleportSettings
    }
  },
  actions: {
    toggleActive() {
      if (this.active) this.reset()
      this.active = !this.active
    },
    setTool(tool: string) {
      this.tool = tool
    },
    setDrawMode(mode: string) {
      this.drawMode = mode
    },
    setEraserMode(mode: string) {
      this.eraserMode = mode
    },
    setSelectedTile(tile: string) {
      this.selectedTile = tile
    },
    setSelectedMapObject(object: MapObject) {
      this.selectedMapObject = object
    },
    toggleSettingsModal() {
      this.isSettingsModalShown = !this.isSettingsModalShown
    },
    toggleMapListModal() {
      this.isMapListModalShown = !this.isMapListModalShown
      this.isCreateMapModalShown = false
    },
    toggleCreateMapModal() {
      this.isCreateMapModalShown = !this.isCreateMapModalShown
    },
    setTeleportSettings(teleportSettings: TeleportSettings) {
      this.teleportSettings = teleportSettings
    },
    triggerClearTiles() {
      this.shouldClearTiles = true
    },
    resetClearTilesFlag() {
      this.shouldClearTiles = false
    },
    reset() {
      this.tool = 'move'
      this.drawMode = 'tile'
      this.selectedTile = ''
      this.selectedMapObject = null
      this.isTileListModalShown = false
      this.isMapObjectListModalShown = false
      this.isSettingsModalShown = false
      this.isMapListModalShown = false
      this.isCreateMapModalShown = false
      this.shouldClearTiles = false
    }
  }
})
